#!/usr/bin/env python

import micropython
from micropython.rsvp import raw
from micropython.graph import get_graph
import rsvp
import sys
import os

code = None

def show(importer, with_builtins=0):
    make(importer, with_builtins)
    show_code(code)

def make(importer, with_builtins=0):
    global code
    code = importer.get_image(with_builtins)

def show_code(code):
    for i, x in enumerate(code):
        print i, x

def show_table_usage(raw_table, slice_size=100):
    for x in xrange(0, len(raw_table), slice_size):
        table_slice = raw_table[x:x+slice_size]
        print "%6d" % (len(table_slice) - table_slice.count(None)), \
            "".join(entry and "#" or "_" for entry in table_slice)

def machine(importer, with_builtins=0, debug=0):
    print "Making the image..."
    make(importer, with_builtins)
    print "Getting raw structures..."
    ot = importer.get_object_table()
    pt = importer.get_parameter_table()
    objlist = ot.as_list()
    paramlist = pt.as_list()
    attr_error = objlist.access("__builtins__", "AttributeError").value.location
    type_error = objlist.access("__builtins__", "TypeError").value.location
    print "Getting raw image..."
    rc = raw(importer.get_image(), ot, pt)
    print "Initialising the machine..."
    rm = rsvp.RSVPMachine(rc, objlist.as_raw(), paramlist.as_raw(), attr_error, type_error, debug=debug)
    rm.pc = importer.code_location
    return rm

def attrs(obj):
    for name, attr in obj.items():
        print name, attr

if __name__ == "__main__":
    args = sys.argv[2:]
    path = sys.path[:]

    if len(sys.argv) > 1:
        program = os.path.abspath(sys.argv[1])
        path.append(os.path.split(program)[0])
    else:
        program = None

    if "-omax" in args:
        requested_optimisations = micropython.Importer.supported_optimisations
    else:
        requested_optimisations = []
        for arg in args:
            if arg.startswith("-o"):
                for arg_part in arg[2:].split(","):
                    requested_optimisations.append(arg_part)

    i = micropython.Importer(path, requested_optimisations, "-v" in args)

    try:
        builtins = i.load_from_file("lib/builtins.py", "__builtins__")
        if program is None:
            print "Loading module micropython ..."
            m = i.load("micropython")
        else:
            print "Loading program", program, "..."
            m = i.load_from_file(program)
    except micropython.ProcessingError, exc:
        print repr(exc)
        if "-tb" in args:
            raise
    else:
        i.vacuum()
        ot = i.get_object_table()
        pt = i.get_parameter_table()

# vim: tabstop=4 expandtab shiftwidth=4
